package authhelper

import (
	"fmt"

	"strings"

	jwt "github.com/dgrijalva/jwt-go"
	"github.com/labstack/echo/middleware"
)

const (
	//JwtKey for key of EP
	JwtKey = "MKCL-EP-1234567890"
)

// JwtCustomClaims are custom claims extending default ones.
type JwtCustomClaims struct {
	Username string `json:"username"`
	jwt.StandardClaims
}

// GetJwtConfig middleware with the custom claims type
func GetJwtConfig() middleware.JWTConfig {
	jwtConfig := middleware.JWTConfig{
		Claims:     &JwtCustomClaims{},
		SigningKey: []byte(JwtKey),
	}
	return jwtConfig
}

// DecodeToken decode token
func DecodeToken(tokenFromRequest string) (jwt.MapClaims, error) {
	tokenFromRequest = strings.Trim(strings.Split(tokenFromRequest, "Bearer")[1], " ")
	// get data i.e.Claims from token
	token, err := jwt.Parse(tokenFromRequest, func(token *jwt.Token) (interface{}, error) {
		// Don't forget to validate the alg is what you expect:
		_, ok := token.Method.(*jwt.SigningMethodRSA)
		if !ok {
			return nil, fmt.Errorf("Unexpected signing method: %v", token.Header["alg"])
		}
		return JwtKey, nil
	})
	if err != nil {
		return nil, err
	}
	claims, ok := token.Claims.(jwt.MapClaims)
	if !ok {
		return nil, nil
	}
	return claims, nil
}
