package downloadhelper

import (
	"flag"
	"io/ioutil"
	"log"
	"net/http"
	"os"
	"strconv"
)

var (
	noOfFiles     = &NoOfFiles
	resume        = NewResume
	maxTryCount   = MaxTryCount
	timeout       = Timeout
	ovrdConnLimit = OvrdConnLimit
)

var (
	// NoOfFiles     = flag.Int("n", 10, "number of parallel connection")
	NoOfFiles        int
	DownloadLocation string
	NewResume        = flag.Bool("r", false, "resume pending download")
	MaxTryCount      = flag.Int("m", 1, "maximum attempts to establish a connection")
	Timeout          = flag.Int("t", 900, "maximum time in seconds it will wait to establish a connection")
	OvrdConnLimit    = flag.Bool("N", false, "maximum connection is restricted to 20, to force more connection")
)

//Download downloads a file from a given url by creating parallel connection
func Download(url string, length int) bool {
	partLength := length / *noOfFiles

	filename := getFilenameFromURL(url)
	filename = getFilename(filename)
	if _, err := os.Stat(DownloadLocation + "/temp/" + filename + "_0"); err == nil {
		log.Fatal("Downloading has already started, resume downloading.")
		return false
	}
	if err := SetupLog(length, *noOfFiles); err != nil {
		log.Fatal(err)
		return false
	}
	for i := 0; i < *noOfFiles; i++ {
		byteStart := partLength * (i)
		byteEnd := byteStart + partLength
		if i == *noOfFiles-1 {
			byteEnd = length
		}
		os.MkdirAll(DownloadLocation+"/temp/", 0777)
		createTempFile(DownloadLocation+"/temp/"+filename+"_"+strconv.Itoa(i), byteStart, byteEnd)
		wg.Add(1)
		go downloadPart(url, filename, i, byteStart, byteEnd)
	}
	wg.Wait()
	FinishLog()
	if !errorGoRoutine {
		mergeFiles(filename, *noOfFiles)
		clearFiles(filename, *noOfFiles)
		log.Println("download successful")
		return true
	}

	log.Println("download unsuccessful")
	return false

}

//Resume resumes a interrupted download by creating same number of connection
func Resume(url string, length int) bool {
	filename := getFilenameFromURL(url)
	filename = getFilename(filename)
	*noOfFiles = noOfExistingConnection(filename, length)

	if *noOfFiles == 0 {
		return false
	}

	partLength := length / *noOfFiles
	if err := SetupResumeLog(filename, length, *noOfFiles); err != nil {
		log.Fatal(err)
		return false
	}
	for i := 0; i < *noOfFiles; i++ {
		partFilename := DownloadLocation + "/temp/" + filename + "_" + strconv.Itoa(i)

		if _, err := os.Stat(partFilename); err != nil {
			byteStart := partLength * (i)
			byteEnd := byteStart + partLength
			if i == *noOfFiles-1 {
				byteEnd = length
			}
			wg.Add(1)
			go downloadPart(url, filename, i, byteStart, byteEnd)
		} else {
			byteStart, byteEnd := readHeader(partFilename)
			if byteStart < byteEnd {
				wg.Add(1)
				go downloadPart(url, filename, i, byteStart, byteEnd)
			}
		}
	}
	wg.Wait()
	FinishLog()
	if !errorGoRoutine {
		mergeFiles(filename, *noOfFiles)
		clearFiles(filename, *noOfFiles)
		log.Println("download successful")
	} else {
		log.Println("download unsuccessful")
	}

	return true
}

//DownloadSingle downloads a file from a given url by creating single connection
func DownloadSingle(url string) bool {
	filename := getFilenameFromURL(url)
	client := &http.Client{}
	req, _ := http.NewRequest("GET", url, nil)
	resp, err := client.Do(req)
	if err != nil {
		log.Fatal(err)
		return false
	}
	reader, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		log.Fatal(err)
		return false
	}
	err = ioutil.WriteFile(filename, reader, 0666)
	if err != nil {
		log.Fatal(err)
		return false
	}
	return true
}
