package securityhelper

import (
	"bytes"
	"crypto/aes"
	"crypto/cipher"
	"encoding/base64"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/coreospackage/filehelper"

	"github.com/OneOfOne/xxhash"
)

const (
	IV = "AAAAAAAAAAAAAAAA"
)

//AESEncrypt Encrypts given text
func AESEncrypt(plainText, key []byte) ([]byte, error) {
	block, err := aes.NewCipher(key)
	if err != nil {
		return nil, err
	}

	padding := block.BlockSize() - len(plainText)%block.BlockSize()

	padtext := bytes.Repeat([]byte{byte(padding)}, padding)

	plainText = append(plainText, padtext...)

	cipherText := make([]byte, len(plainText))

	cbc := cipher.NewCBCEncrypter(block, []byte(IV))
	cbc.CryptBlocks(cipherText, plainText)

	encodedData := make([]byte, base64.StdEncoding.EncodedLen(len(plainText)))

	base64.StdEncoding.Encode(encodedData, cipherText)

	return encodedData, nil
}

//AESDecrypt Decrypts given cipher text
func AESDecrypt(encodedData, key []byte) []byte {
	decodedData := make([]byte, base64.StdEncoding.DecodedLen(len(encodedData)))
	n, err := base64.StdEncoding.Decode(decodedData, encodedData)
	if err != nil {
		panic(err)
	}

	cipherText := decodedData[:n]

	block, err := aes.NewCipher(key)
	if err != nil {
		panic(err)
	}

	cbc := cipher.NewCBCDecrypter(block, []byte(IV))
	cbc.CryptBlocks(cipherText, cipherText)

	length := len(cipherText)

	unpadding := int(cipherText[length-1])

	return cipherText[:(length - unpadding)]
}

//GetHashChecksumOfByteArray Get Hash Check sum
func GetHashChecksumOfByteArray(byteArray []byte) (uint64, error) {
	h := xxhash.New64()
	_, err := h.Write(byteArray)
	if err != nil {
		return 0, err
	}
	return h.Sum64(), nil
}

//GetHashChecksumOfFile Get Hash Check sum
func GetHashChecksumOfFile(filePath string) (uint64, error) {
	byteArray, err := filehelper.ReadFile(filePath)
	if err != nil {
		return 0, err
	}
	hash, err := GetHashChecksumOfByteArray(byteArray)
	if err != nil {
		return 0, err
	}
	return hash, nil
}
